@@ -72,6 +72,7 @@ int rawDelay = 0;
 int copyRectDelay = 0;
 
 Bool debug = False;
+Bool trimsel = False;
 
 void usage()
 {
@@ -86,11 +87,21 @@ void usage()
 	  "       %s [<options>] -listen [<display#>]\n"
 	  "\n"
 	  "<options> are:\n"
-	  "              [-display <display>] [-shared]\n"
+	  "              [-display <display>]\n"
+          "              [-version]\n"
+          "              [-shared]\n"
 	  "              [-north] [-south] [-east] [-west]\n"
-	  "              [-passwdfile <passwd-file>] [-version]\n"
-	  "              [-resurface] [-edgewidth width]\n"
-	  "              [-nowheel] [-reversewheel]\n"
+          "              [-hotkey key]\n"
+	  "              [-passwdfile <passwd-file>]\n"
+	  "              [-resurface]\n"
+	  "              [-edgewidth width]\n"
+	  "              [-desktop desktop]\n"
+	  "              [-timeout seconds]\n"
+	  "              [-nowheel]\n"
+	  "              [-reversewheel]\n"
+	  "              [-scrolllines lines]\n"
+	  "              [-mac]\n"
+	  "              [-trimsel]\n"
 	  ,programName,programName);
   exit(1);
 }
@@ -118,6 +129,9 @@ void processArgs(int argc, char **argv)
       edge=EDGE_NORTH;
     } else if (strcmp(argv[i],"-south") == 0) {
       edge=EDGE_SOUTH;
+    } else if (strcmp(argv[i],"-mac") == 0) {
+      extern int mac_mode;
+      mac_mode=1;
     } else if (strcmp(argv[i],"-nowheel") == 0) {
       extern int emulate_wheel;
       emulate_wheel=0;
@@ -137,6 +151,32 @@ void processArgs(int argc, char **argv)
 	fprintf(stderr,"x2vnc: -edgewidth cannot be less than 1\n");
 	exit(1);
       }
+    } else if (strcmp(argv[i],"-desktop") == 0) {
+      extern int requested_desktop;
+      if (++i >= argc) usage();
+      requested_desktop = atoi(argv[i]);
+    } else if (strcmp(argv[i],"-timeout") == 0) {
+      extern long grab_timeout_delay;
+      if (++i >= argc) usage();
+      grab_timeout_delay = atoi(argv[i]);
+      if(grab_timeout_delay < 0)
+      {
+	fprintf(stderr,"x2vnc: -timeout cannot be less than 0\n");
+	exit(1);
+      }
+    } else if (strcmp(argv[i],"-scrolllines") == 0) {
+      extern int scroll_lines;
+      if (++i >= argc) usage();
+      scroll_lines = atoi(argv[i]);
+      if(scroll_lines < 1)
+      {
+	fprintf(stderr,"x2vnc: -scrollines cannot be less than 1\n");
+	exit(1);
+      }
+    } else if (strcmp(argv[i],"-hotkey") == 0) {
+      extern void sethotkey(char *);
+      if (++i >= argc) usage();
+      sethotkey(argv[i]);
     } else if (strcmp(argv[i],"-passwd") == 0) {
       if (++i >= argc) usage();
       passwdFile = argv[i];
@@ -145,6 +185,8 @@ void processArgs(int argc, char **argv)
       passwdFile = argv[i];
     } else if (strcmp(argv[i],"-debug") == 0) {
       debug = True;
+    } else if (strcmp(argv[i],"-trimsel") == 0) {
+      trimsel = True;
     } else if (strcmp(argv[i],"-listen") == 0) {
       if (argumentSpecified) usage();
       
