@@ -368,6 +368,41 @@ SendKeyEvent(CARD32 key, Bool down)
     return WriteExact(rfbsock, (char *)&ke, sz_rfbKeyEventMsg);
 }
 
+/*
+ * Returns true if STR looks like a URL.
+ * Note that STR is not neccessarily NUL-terminated.
+ */
+static Bool
+LooksLikeURL(const char *str, int len)
+{
+    /*
+     * Skip leading whitespace.
+     */
+    while (len != 0 && isspace(*str)) {
+	str++, len--;
+    }
+    if (len == 0) {
+	return False;
+    }
+
+    /*
+     * Check for scheme:// according to rfc2396
+     *    [A-Za-z][A-Za-z0-9.+-]*://
+     */
+    if (isalpha(*str)) {
+	str++, len--;
+	while (len != 0 && 
+	       (isalpha(*str) || isdigit(*str) ||
+	        *str == '.' || *str == '+' || *str == '-')) {
+	    str++, len--;
+	}
+	if (len >= 3 && str[0] == ':' && str[1] == '/' && str[2] == '/') {
+	    return True;
+	}
+    }
+
+    return False;
+}
 
 /*
  * SendClientCutText.
@@ -378,6 +413,20 @@ SendClientCutText(char *str, int len)
 {
     rfbClientCutTextMsg cct;
 
+    if (trimsel && LooksLikeURL(str, len)) {
+	/*
+	 * Trim leading and trailing spaces.
+	 */
+	while (len != 0 && isspace(str[0])) {
+	    str++;
+	    len--;
+	}
+	while (len != 0 && isspace(str[len-1])) {
+	    str[len-1] = '\0';
+	    len--;
+	}
+    }
+
     cct.type = rfbClientCutText;
     cct.length = Swap32IfLE(len);
     return  (WriteExact(rfbsock, (char *)&cct, sz_rfbClientCutTextMsg) &&
