@@ -52,10 +52,37 @@ int restingx=-1;
 int restingy=-1;
 int emulate_wheel=1;
 int wheel_button_up=4;
+int scroll_lines=1;
+int mac_mode=0;
+int hidden=0;
+
+KeySym grabkeysym=XK_F12;
+KeyCode grabkey;
+int grabmod = ControlMask;
 
 char *client_selection_text=0;
 size_t client_selection_text_length=0;
 
+int saved_xpos=-1;
+int saved_ypos=-1;
+
+int saved_remote_xpos=-1;
+int saved_remote_ypos=-1;
+
+long grab_timeout=0;
+long grab_timeout_delay=590;
+#define SET_GRAB_TIMEOUT() grab_timeout= time(0) + grab_timeout_delay
+
+
+/* We must do our own desktop handling */
+Atom current_desktop_atom;
+Atom number_of_desktops_atom;
+
+int requested_desktop = -2;
+int current_desktop = -2;
+int current_number_of_desktops = -2;
+
+static void ungrabit(int x, int y, Window warpWindow);
 
 /*
  * This variable is true (1) if the mouse is on the same screen as the one
@@ -70,140 +97,181 @@ static Bool mouseOnScreen;
 
 Bool CreateXWindow(void)
 {
-    XSetWindowAttributes attr;
-    XEvent ev;
-    char defaultGeometry[256];
-    XSizeHints wmHints;
-    XGCValues gcv;
-    int i;
+  XSetWindowAttributes attr;
+  XEvent ev;
+  char defaultGeometry[256];
+  XSizeHints wmHints;
+  XGCValues gcv;
+  int i;
+  
+  Pixmap    nullPixmap;
+  XColor    dummyColor;
+  
+  if (!(dpy = XOpenDisplay(displayname))) {
+    fprintf(stderr,"%s: unable to open display %s\n",
+	    programName, XDisplayName(displayname));
+    return False;
+  }
+  
+  for (i = 0; i < 256; i++)
+    modifierPressed[i] = False;
+  
+  /* Try to work out the geometry of the top-level window */
+  
+  displayWidth = WidthOfScreen(DefaultScreenOfDisplay(dpy));
+  displayHeight = HeightOfScreen(DefaultScreenOfDisplay(dpy));
+  
+  saved_remote_xpos = displayWidth / 2;
+  saved_remote_ypos = displayHeight / 2;
+  
+  if(restingy == -1)
+  {
+    restingy = si.framebufferHeight -2;
+    restingx = si.framebufferWidth -2;
+  }
 
-    Pixmap    nullPixmap;
-    XColor    dummyColor;
-
-    if (!(dpy = XOpenDisplay(displayname))) {
-	fprintf(stderr,"%s: unable to open display %s\n",
-		programName, XDisplayName(displayname));
-	return False;
-    }
-
-    for (i = 0; i < 256; i++)
-	modifierPressed[i] = False;
-
-    /* Try to work out the geometry of the top-level window */
-
-    displayWidth = WidthOfScreen(DefaultScreenOfDisplay(dpy));
-    displayHeight = HeightOfScreen(DefaultScreenOfDisplay(dpy));
-
-    if(restingy == -1)
-    {
-      restingy = si.framebufferHeight -2;
-      restingx = si.framebufferWidth -2;
-    }
-
-    topLevelWidth=edge_width;
-    topLevelHeight=edge_width;
-    wmHints.x=0;
-    wmHints.y=0;
-
-    switch(edge)
-    {
-      case EDGE_EAST: wmHints.x=displayWidth-edge_width;
-      case EDGE_WEST: topLevelHeight=displayHeight;
-	break;
-
-      case EDGE_SOUTH: wmHints.y=displayHeight-edge_width;
-      case EDGE_NORTH: topLevelWidth=displayWidth;
-	break;
-    }
-
-    wmHints.flags = PMaxSize | PMinSize |PPosition |PBaseSize;
-
-    wmHints.min_width = topLevelWidth;
-    wmHints.min_height = topLevelHeight;
-
-    wmHints.max_width = topLevelWidth;
-    wmHints.max_height = topLevelHeight;
-
-    wmHints.base_width = topLevelWidth;
-    wmHints.base_height = topLevelHeight;
-
-    sprintf(defaultGeometry, "%dx%d+%d+%d",
-	    topLevelWidth, topLevelHeight,
-	    wmHints.x, wmHints.y);
-
-    XWMGeometry(dpy, DefaultScreen(dpy), geometry, defaultGeometry, 0,
-		&wmHints, &wmHints.x, &wmHints.y,
-		&topLevelWidth, &topLevelHeight, &wmHints.win_gravity);
-
-    /* Create the top-level window */
-
-    attr.border_pixel = 0; /* needed to allow 8-bit cmap on 24-bit display -
-			      otherwise we get a Match error! */
-    attr.background_pixel = BlackPixelOfScreen(DefaultScreenOfDisplay(dpy));
-    attr.event_mask = ( LeaveWindowMask|
-			StructureNotifyMask|
-			ButtonPressMask|
-			ButtonReleaseMask|
-			PointerMotionMask|
-			KeyPressMask|
-			KeyReleaseMask|
-			EnterWindowMask|
-			(resurface?VisibilityChangeMask:0) );
+  topLevelWidth=edge_width;
+  topLevelHeight=edge_width;
+  wmHints.x=0;
+  wmHints.y=0;
+  
+  switch(edge)
+  {
+    case EDGE_EAST: wmHints.x=displayWidth-edge_width;
+    case EDGE_WEST: topLevelHeight=displayHeight;
+      break;
       
-    attr.override_redirect=True;
+    case EDGE_SOUTH: wmHints.y=displayHeight-edge_width;
+    case EDGE_NORTH: topLevelWidth=displayWidth;
+      break;
+  }
+  
+  wmHints.flags = PMaxSize | PMinSize |PPosition |PBaseSize;
+  
+  wmHints.min_width = topLevelWidth;
+  wmHints.min_height = topLevelHeight;
+  
+  wmHints.max_width = topLevelWidth;
+  wmHints.max_height = topLevelHeight;
+  
+  wmHints.base_width = topLevelWidth;
+  wmHints.base_height = topLevelHeight;
+  
+  sprintf(defaultGeometry, "%dx%d+%d+%d",
+	  topLevelWidth, topLevelHeight,
+	  wmHints.x, wmHints.y);
+  
+  XWMGeometry(dpy, DefaultScreen(dpy), geometry, defaultGeometry, 0,
+	      &wmHints, &wmHints.x, &wmHints.y,
+	      &topLevelWidth, &topLevelHeight, &wmHints.win_gravity);
+  
+  /* Create the top-level window */
+  
+  attr.border_pixel = 0; /* needed to allow 8-bit cmap on 24-bit display -
+			    otherwise we get a Match error! */
+  attr.background_pixel = BlackPixelOfScreen(DefaultScreenOfDisplay(dpy));
+  attr.event_mask = ( LeaveWindowMask|
+		      StructureNotifyMask|
+		      ButtonPressMask|
+		      ButtonReleaseMask|
+		      PointerMotionMask|
+		      KeyPressMask|
+		      KeyReleaseMask|
+		      EnterWindowMask|
+		      (resurface?VisibilityChangeMask:0) );
+  
+  attr.override_redirect=1;
 
-    topLevel = XCreateWindow(dpy, DefaultRootWindow(dpy), wmHints.x, wmHints.y,
-			     topLevelWidth, topLevelHeight, 0, CopyFromParent,
-			     InputOutput, CopyFromParent,
-			     (CWBorderPixel|
-			      CWEventMask|
-			      CWOverrideRedirect|
-			      CWBackPixel),
-			     &attr);
+  topLevel = XCreateWindow(dpy, DefaultRootWindow(dpy), wmHints.x, wmHints.y,
+			   topLevelWidth, topLevelHeight, 0, CopyFromParent,
+			   InputOutput, CopyFromParent,
+			   (CWBorderPixel|
+			    CWEventMask|
+			    CWOverrideRedirect|
+			    CWBackPixel),
+			   &attr);
 
-    wmHints.flags |= USPosition; /* try to force WM to place window */
-    XSetWMNormalHints(dpy, topLevel, &wmHints);
+  current_desktop_atom = XInternAtom(dpy, "_NET_CURRENT_DESKTOP", False);
+  number_of_desktops_atom = XInternAtom(dpy, "_NET_NUMBER_OF_DESKTOPS", False);
 
-    wmProtocols = XInternAtom(dpy, "WM_PROTOCOLS", False);
-    wmDeleteWindow = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
-    XSetWMProtocols(dpy, topLevel, &wmDeleteWindow, 1);
+#if 1
+  {
+    Atom t = XInternAtom(dpy, "_NET_WM_WINDOW_DOCK", False);
 
-    XStoreName(dpy, topLevel, desktopName);
+    XChangeProperty(dpy,
+		    topLevel,
+		    XInternAtom(dpy, "_NET_WM_WINDOW_TYPE", False),
+		    XA_ATOM,
+		    32,
+		    PropModeReplace,
+		    (char *)&t,
+		    1);
+  }
+#endif
 
 
-    XMapRaised(dpy, topLevel);
+  wmHints.flags |= USPosition; /* try to force WM to place window */
+  XSetWMNormalHints(dpy, topLevel, &wmHints);
 
-    /*
-     * For multi-screen setups, we need to know if the mouse is on the 
-     * screen.
-     * - GRM
-     */
-    if (ScreenCount(dpy) > 1) {
-      Window root, child;
-      int root_x, root_y;
-      int win_x, win_y;
-      unsigned int keys_buttons;
-
-      XSelectInput(dpy, DefaultRootWindow(dpy), PropertyChangeMask |
-                   EnterWindowMask | LeaveWindowMask);
-        /* Cut buffer happens on screen 0 only. */
-      if (DefaultRootWindow(dpy) != RootWindow(dpy, 0)) {
-          XSelectInput(dpy, RootWindow(dpy, 0), PropertyChangeMask);
-      }
-      XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &child,
-                    &root_x, &root_y, &win_x, &win_y, &keys_buttons);
-      mouseOnScreen = root == DefaultRootWindow(dpy);
-    } else {
-      XSelectInput(dpy, DefaultRootWindow(dpy), PropertyChangeMask);
-      mouseOnScreen = 1;
-    }
-
-    nullPixmap = XCreatePixmap(dpy, DefaultRootWindow(dpy), 1, 1, 1);
-    grabCursor = 
-      XCreatePixmapCursor(dpy, nullPixmap, nullPixmap,
-			  &dummyColor, &dummyColor, 0, 0);
+  wmProtocols = XInternAtom(dpy, "WM_PROTOCOLS", False);
+  wmDeleteWindow = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
+  XSetWMProtocols(dpy, topLevel, &wmDeleteWindow, 1);
+  
+  XStoreName(dpy, topLevel, desktopName);
+  
+  XMapRaised(dpy, topLevel);
+  
+  /*
+   * For multi-screen setups, we need to know if the mouse is on the 
+   * screen.
+   * - GRM
+   */
+  if (ScreenCount(dpy) > 1) {
+    Window root, child;
+    int root_x, root_y;
+    int win_x, win_y;
+    unsigned int keys_buttons;
     
-    return True;
+    XSelectInput(dpy, DefaultRootWindow(dpy), PropertyChangeMask |
+		 EnterWindowMask | LeaveWindowMask | KeyPressMask);
+    /* Cut buffer happens on screen 0 only. */
+    if (DefaultRootWindow(dpy) != RootWindow(dpy, 0)) {
+      XSelectInput(dpy, RootWindow(dpy, 0), PropertyChangeMask);
+    }
+    XQueryPointer(dpy, DefaultRootWindow(dpy), &root, &child,
+		  &root_x, &root_y, &win_x, &win_y, &keys_buttons);
+    mouseOnScreen = root == DefaultRootWindow(dpy);
+  } else {
+    XSelectInput(dpy, DefaultRootWindow(dpy), PropertyChangeMask);
+    mouseOnScreen = 1;
+  }
+  
+  nullPixmap = XCreatePixmap(dpy, DefaultRootWindow(dpy), 1, 1, 1);
+  grabCursor = 
+    XCreatePixmapCursor(dpy, nullPixmap, nullPixmap,
+			&dummyColor, &dummyColor, 0, 0);
+  
+  if(grabkeysym)
+  {
+    grabkey=XKeysymToKeycode(dpy, grabkeysym);
+#ifdef DEBUG
+    fprintf(stderr,"Grabbing key = %d, modifiers = %x\n",grabkey,grabmod);
+#endif
+    XGrabKey(dpy, grabkey, grabmod,
+	     DefaultRootWindow(dpy),
+	     1, GrabModeSync, GrabModeSync);
+
+    /* Allow numlock */
+    XGrabKey(dpy, grabkey, grabmod | Mod2Mask,
+	     DefaultRootWindow(dpy),
+	     1, GrabModeSync, GrabModeSync);
+  }
+
+  /* hide the cursor, so user won't get confused
+     which keyboard has control */
+  SendPointerEvent(restingx,restingy,0);
+  
+  return True;
 }
 
 
@@ -215,7 +283,7 @@ Bool CreateXWindow(void)
 void
 ShutdownX()
 {
-    XCloseDisplay(dpy);
+  XCloseDisplay(dpy);
 }
 
 
@@ -223,9 +291,14 @@ ShutdownX()
  * HandleXEvents.
  */
 
-Bool HandleXEvents()
+Bool HandleXEvents(void)
 {
   XEvent ev;
+
+  if(grab_timeout_delay && grabbed && time(0) > grab_timeout)
+  {
+    ungrabit(-1, -1, DefaultRootWindow(dpy));
+  }
   
   /* presumably XCheckMaskEvent is more efficient than XCheckIfEvent -GRM */
     while (XCheckIfEvent(dpy, &ev, AllXEventsPredicate, NULL))
@@ -275,13 +348,24 @@ static int leave_translate(int isedge, int width, int pos)
 #define EDGE(X) ((X)?edge_width:0)
 
 #define SCALEX(X) \
-( ((X)-EDGE(edge==EDGE_WEST ))*(si.framebufferWidth-1 )/(displayWidth -EDGE(EW)) )
+( ((X)-EDGE(edge==EDGE_WEST ))*(si.framebufferWidth-1 )/(displayWidth -1-EDGE(EW)) )
 
 #define SCALEY(Y) \
-( ((Y)-EDGE(edge==EDGE_NORTH))*(si.framebufferHeight-1)/(displayHeight-EDGE(NS)) )
+( ((Y)-EDGE(edge==EDGE_NORTH))*(si.framebufferHeight-1)/(displayHeight-1-EDGE(NS)) )
 
 
-static void grabit(XEvent *ev)
+static int sendpointerevent(int localx, int localy, int buttonmask)
+{
+  if(mac_mode && (buttonmask & (1<<2)))
+  {
+    buttonmask &=~ (1<<2);
+    buttonmask |=~ (1<<0);
+  }
+
+  return SendPointerEvent(SCALEX(localx), SCALEY(localy), buttonmask);
+}
+
+static void grabit(int x, int y, int state)
 {
   Window selection_owner;
 
@@ -293,13 +377,13 @@ static void grabit(XEvent *ev)
 		GrabModeAsync, GrabModeAsync,
 		CurrentTime);
   XFlush(dpy);
-  
-  XWarpPointer(dpy,None, DefaultRootWindow(dpy),0,0,0,0,
-	       enter_translate(EW,displayWidth ,ev->xcrossing.x_root),
-	       enter_translate(NS,displayHeight,ev->xcrossing.y_root));
-  
-  SendPointerEvent(SCALEX(ev->xcrossing.x_root), SCALEY(ev->xcrossing.y_root),
-		   (ev->xcrossing.state & 0x1f00) >> 8);
+
+  if(x > -1 && y > -1)
+  {
+    XWarpPointer(dpy,None, DefaultRootWindow(dpy),0,0,0,0, x, y);
+    sendpointerevent(x, y, (state & 0x1f00) >> 8);
+  }
+
   grabbed=1;
   mouseOnScreen = 1;
 
@@ -312,6 +396,30 @@ static void grabit(XEvent *ev)
 		      XA_PRIMARY, XA_STRING, XA_CUT_BUFFER0,
 		      topLevel, CurrentTime);
   }
+  SET_GRAB_TIMEOUT();
+}
+
+static void ungrabit(int x, int y, Window warpWindow)
+{
+  int i;
+  SendPointerEvent(restingx,restingy,0);
+  if(x > -1 && y > -1 )
+    XWarpPointer(dpy,None, warpWindow, 0,0,0,0, x, y);
+  XUngrabKeyboard(dpy, CurrentTime);
+  XUngrabPointer(dpy, CurrentTime);
+  mouseOnScreen = warpWindow == DefaultRootWindow(dpy);
+  XFlush(dpy);
+  
+  for (i = 255; i >= 0; i--)
+  {
+    if (modifierPressed[i]) {
+      if (!SendKeyEvent(XKeycodeToKeysym(dpy, i, 0), False))
+	return;
+      modifierPressed[i]=False;
+    }
+  }
+  
+  grabbed=0;
 }
 
 /*
@@ -336,6 +444,8 @@ static Bool HandleTopLevelEvent(XEvent *ev)
   int buttonMask;
   KeySym ks;
   
+  SET_GRAB_TIMEOUT();
+
   switch (ev->type)
   {
     case SelectionRequest:
@@ -422,7 +532,8 @@ static Bool HandleTopLevelEvent(XEvent *ev)
        * Please tell me if you have a problem with this.
        * - Hubbe
        */
-      if (ev->xvisibility.state == VisibilityPartiallyObscured && resurface)
+      if (ev->xvisibility.state == VisibilityPartiallyObscured &&
+	  resurface && !hidden)
       {
 	static long last_resurface=0;
 	long t=time(0);
@@ -431,7 +542,9 @@ static Bool HandleTopLevelEvent(XEvent *ev)
 	  shortsleep(5000);
 
 	last_resurface=t;
-	  
+#ifdef DEBUG
+	fprintf(stderr,"Raising window!\n");
+#endif
 	XRaiseWindow(dpy, topLevel);
       }
       return 1;
@@ -444,7 +557,9 @@ static Bool HandleTopLevelEvent(XEvent *ev)
     case EnterNotify:
       if(!grabbed && ev->xcrossing.mode==NotifyNormal)
       {
-	grabit(ev);
+	grabit(enter_translate(EW,displayWidth ,ev->xcrossing.x_root),
+	       enter_translate(NS,displayHeight,ev->xcrossing.y_root),
+	       ev->xcrossing.state);
       }
       return 1;
       
@@ -457,9 +572,15 @@ static Bool HandleTopLevelEvent(XEvent *ev)
         Window warpWindow;
         while (XCheckTypedWindowEvent(dpy, topLevel, MotionNotify, ev))
 	  ;	/* discard all queued motion notify events */
-	
-	i=SendPointerEvent(SCALEX(ev->xmotion.x_root),
-			   SCALEY(ev->xmotion.y_root),
+
+#if 0
+	fprintf(stderr,"{ %d, %d } -> { %d, %d }\n",
+		ev->xmotion.x_root,
+		ev->xmotion.y_root,
+		SCALEX(ev->xmotion.x_root),
+		SCALEY(ev->xmotion.y_root));
+#endif
+	i=sendpointerevent(ev->xmotion.x_root, ev->xmotion.y_root,
 			   (ev->xmotion.state & 0x1f00) >> 8);
 
 	if(ev->xmotion.state & 0x1f00) return 1;
@@ -541,25 +662,9 @@ static Bool HandleTopLevelEvent(XEvent *ev)
           
 	if(d)
 	{
-	  SendPointerEvent(restingx,restingy,0);
-	  XWarpPointer(dpy,None, warpWindow, 0,0,0,0, x, y);
-	  XUngrabKeyboard(dpy, CurrentTime);
-	  XUngrabPointer(dpy, CurrentTime);
-          mouseOnScreen = warpWindow == DefaultRootWindow(dpy);
-	  XFlush(dpy);
-	  
-	  for (i = 255; i >= 0; i--)
-	  {
-	    if (modifierPressed[i]) {
-	      if (!SendKeyEvent(XKeycodeToKeysym(dpy, i, 0), False))
-		return False;
-	      modifierPressed[i]=False;
-	    }
-	  }
-	  
-	  grabbed=0;
+	  ungrabit(x, y, warpWindow);
+	  return 1;
 	}
-	return i;
       }
       return 1;
       
@@ -567,18 +672,32 @@ static Bool HandleTopLevelEvent(XEvent *ev)
     case ButtonRelease:
       if (ev->xbutton.button > 3 && emulate_wheel)
       {
+	int l;
 	if (ev->xbutton.button == wheel_button_up)
 	  ks = XK_Up;
 	else
 	  ks = XK_Down;
 
-	if (ev->type == ButtonPress)
-	  SendKeyEvent(ks, 1); /* keydown */
-	else
-	  SendKeyEvent(ks, 0); /* keyup */
+	if(ev->type == ButtonPress)
+	{
+	  for(l=1;l<scroll_lines;l++)
+	  {
+	    SendKeyEvent(ks, 1); /* keydown */
+	    SendKeyEvent(ks, 0); /* keyup */
+	  }
+	}
+
+	SendKeyEvent(ks, ev->type == ButtonPress); /* keydown */
 	break;
       }
 
+      if(mac_mode && ev->xbutton.button == 3)
+      {
+	int ctrlcode=XKeysymToKeycode(dpy, XK_Control_L);
+	SendKeyEvent(XK_Control_L, ev->type == ButtonPress);
+	modifierPressed[ctrlcode]=ev->type == ButtonPress;
+      }
+
       if (ev->type == ButtonPress) {
 	buttonMask = (((ev->xbutton.state & 0x1f00) >> 8) |
 		      (1 << (ev->xbutton.button - 1)));
@@ -587,8 +706,8 @@ static Bool HandleTopLevelEvent(XEvent *ev)
 		      ~(1 << (ev->xbutton.button - 1)));
       }
       
-      return SendPointerEvent(SCALEX(ev->xbutton.x_root),
-			      SCALEY(ev->xbutton.y_root),
+      return sendpointerevent(ev->xbutton.x_root,
+			      ev->xbutton.y_root,
 			      buttonMask);
 	
     case KeyPress:
@@ -596,11 +715,27 @@ static Bool HandleTopLevelEvent(XEvent *ev)
     {
       char keyname[256];
       keyname[0] = '\0';
+
+
       XLookupString(&ev->xkey, keyname, 256, &ks, NULL);
 /*      fprintf(stderr,"Pressing %x (%c) name=%s  code=%d\n",ks,ks,keyname,ev->xkey.keycode); */
 
+      if(ev->type == KeyPress && 
+	 ev->xkey.keycode == grabkey &&
+	 (ev->xkey.state == grabmod ||
+	  (ev->xkey.state & ~ Mod2Mask ) == grabmod ))
+      {
+	saved_remote_xpos=ev->xkey.x_root;
+	saved_remote_ypos=ev->xkey.y_root;
+	ungrabit(saved_xpos,saved_ypos,DefaultRootWindow(dpy));
+	return 1;
+      }
       if (IsModifierKey(ks)) {
 	ks = XKeycodeToKeysym(dpy, ev->xkey.keycode, 0);
+
+	/* Ignore AltGr key, it is handled by XLookupString */
+	if( ks == XK_Mode_switch ) return True;
+
 	modifierPressed[ev->xkey.keycode] = (ev->type == KeyPress);
       } else {
 	/* This fixes the 'shift-tab' problem - Hubbe */
@@ -630,7 +765,67 @@ static Bool HandleTopLevelEvent(XEvent *ev)
     return True;
 }
 
+int get_root_int_prop(Atom property)
+{
+  unsigned char *data=0;
+  Atom t;
+  int format;
+  unsigned long len;
+  unsigned long bytes_left;
+  int ret=XGetWindowProperty(dpy,
+			     DefaultRootWindow(dpy),
+			     property,
+			     0,
+			     1,
+			     0, /* delete */
+			     XA_CARDINAL,
+			     &t,
+			     &format,
+			     &len,
+			     &bytes_left,
+			     &data);
 
+  ret = *(CARD32 *)data;
+  
+  if(data)
+    XFree(data);
+
+  return ret;
+}
+
+void mapwindow(void)
+{
+  hidden=0;
+  XMapRaised(dpy, topLevel);
+}
+
+void hidewindow(void)
+{
+  hidden=1;
+  XUnmapWindow(dpy, topLevel);
+}
+
+void check_desktop(void)
+{
+  int t=requested_desktop;
+
+  current_desktop=get_root_int_prop(current_desktop_atom);
+  
+  switch(t)
+  {
+    case -2: t=current_desktop; break;
+    case -1:
+      t=get_root_int_prop(number_of_desktops_atom)-1;
+      if(t<0) t=0;
+      break;
+  }
+  if( t == current_desktop)
+  {
+    mapwindow();
+  }else{
+    hidewindow();
+  }
+}
 
 /*
  * HandleRootEvent.
@@ -648,6 +843,24 @@ static Bool HandleRootEvent(XEvent *ev)
   
   switch (ev->type)
   {
+    case KeyPress:
+    {
+      KeySym ks;
+      char keyname[256];
+      keyname[0] = '\0';
+      XLookupString(&ev->xkey, keyname, 256, &ks, NULL);
+#ifdef DEBUG
+      fprintf(stderr,"ROOT: Pressing %x (%c) name=%s  code=%d\n",ks,ks,keyname,ev->xkey.keycode);
+#endif
+      if(ev->xkey.keycode)
+      {
+	saved_xpos=ev->xkey.x_root;
+	saved_ypos=ev->xkey.y_root;
+	grabit(saved_remote_xpos, saved_remote_ypos, 0);
+      }
+      break;
+    }
+
     case EnterNotify:
     case LeaveNotify:
       /*
@@ -707,7 +920,9 @@ static Bool HandleRootEvent(XEvent *ev)
        */
       if(grab && ev->xcrossing.mode == NotifyNormal)
       {
-	grabit(ev);
+	grabit(enter_translate(EW,displayWidth ,ev->xcrossing.x_root),
+	       enter_translate(NS,displayHeight,ev->xcrossing.y_root),
+	       ev->xcrossing.state);
       }
       break;
       
@@ -717,13 +932,17 @@ static Bool HandleRootEvent(XEvent *ev)
 	str = XFetchBytes(dpy, &len);
 	if (str) {
 #ifdef DEBUG
-	  fprintf(stderr,"GOT CUT TEXT: %s\n",str);
+	  fprintf(stderr,"GOT CUT TEXT: \"%s\"\n",str);
 #endif
 	  if (!SendClientCutText(str, len))
 	    return False;
 	  XFree(str);
 	}
       }
+      if(ev->xproperty.atom == current_desktop_atom ||
+	 ev->xproperty.atom == number_of_desktops_atom)
+	check_desktop();
+
       break;
   }
   
@@ -761,4 +980,44 @@ AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
 }
 
 
+void sethotkey(char *key)
+{
+  grabmod = 0;
 
+#define GOBBLE(X,Y)						\
+    if(!strncasecmp(key,X "-",sizeof(X) +1 -sizeof(""))) {	\
+      grabmod|=Y;						\
+      key+=sizeof(X) +1 -sizeof("");				\
+      continue;							\
+    }
+
+  while(1)
+  {
+    GOBBLE("s",ShiftMask);
+    GOBBLE("shift",ShiftMask);
+
+    GOBBLE("c",ControlMask);
+    GOBBLE("ctrl",ControlMask);
+    GOBBLE("control",ControlMask);
+
+    GOBBLE("a",Mod1Mask);
+    GOBBLE("alt",Mod1Mask);
+    GOBBLE("mod1",Mod1Mask);
+
+    GOBBLE("mod2",Mod2Mask);
+
+    GOBBLE("m",Mod3Mask);
+    GOBBLE("meta",Mod3Mask);
+    GOBBLE("mod3",Mod3Mask);
+
+    GOBBLE("super",Mod4Mask);
+    GOBBLE("mod4",Mod4Mask);
+
+    GOBBLE("hyper",Mod5Mask);
+    GOBBLE("mod5",Mod5Mask);
+
+    break;
+  }
+
+  grabkeysym=XStringToKeysym(key);
+}
